package com.example.anchorpoints;

import android.app.Activity;
import android.os.Bundle;
import android.view.Menu;

import com.mindfusion.diagramming.AeroEffect;
import com.mindfusion.diagramming.AnchorPattern;
import com.mindfusion.diagramming.AnchorPoint;
import com.mindfusion.diagramming.Diagram;
import com.mindfusion.diagramming.DiagramAdapter;
import com.mindfusion.diagramming.DiagramView;
import com.mindfusion.diagramming.GlassEffect;
import com.mindfusion.diagramming.MarkStyle;
import com.mindfusion.diagramming.NodeEvent;
import com.mindfusion.diagramming.QuickRouter;
import com.mindfusion.diagramming.Shape;
import com.mindfusion.diagramming.ShapeNode;
import com.mindfusion.diagramming.ShowAnchors;
import com.mindfusion.diagramming.SnapToAnchor;
import com.mindfusion.drawing.Color;
import com.mindfusion.drawing.GradientBrush;
import com.mindfusion.drawing.SolidBrush;

public class MainActivity extends Activity
{
	private DiagramView diagramView;
	private Diagram diagram;

	static AnchorPattern apat1, apat2;

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
		diagramView = (DiagramView)findViewById(R.id.diag_view);
		diagram = diagramView.getDiagram();

		initDiagram();
		if (savedInstanceState == null)
			createItems();
		// else the diagram is restored automatically from state
	}

    private void initDiagram()
	{
    	// set up anchor point options
    	diagram.setSnapToAnchor(SnapToAnchor.OnCreateOrModify);
    	diagram.setShowAnchors(ShowAnchors.Always);

    	// enable automatic link routing
    	QuickRouter router = (QuickRouter)diagram.getLinkRouter();
    	router.setUBendMaxLen(10);
    	diagram.setRouteLinks(true);

    	// customize appearance of nodes
    	diagram.getNodeEffects().add(new GlassEffect());
		diagram.getNodeEffects().add(new AeroEffect());
		
		diagram.getShapeNodeStyle().setBrush(
			new GradientBrush(Color.lightGray, new Color(0, 0xDE, 0xD1), 30));
		diagram.getDiagramStyle().setShadowBrush(
			new SolidBrush(new Color(0xBE, 0xBE, 0xBE, 0x96)));
		
		// define custom anchor point patterns
		if (apat1 == null)
		{
			apat1 = new AnchorPattern(new AnchorPoint[]
			{
				new AnchorPoint(50, 0, true, true),
				new AnchorPoint(100, 50, true, true),
				new AnchorPoint(50, 100, true, true),
				new AnchorPoint(0, 50, true, true)
			}, "rhombus1");
	
			apat2 = new AnchorPattern(new AnchorPoint[]
			{
				new AnchorPoint(0, 0, true, false, MarkStyle.Circle, Color.blue),
				new AnchorPoint(50, 0, true, false, MarkStyle.Circle, Color.blue),
				new AnchorPoint(100, 0, true, false, MarkStyle.Circle, Color.red),
				new AnchorPoint(00, 100, false, true, MarkStyle.Rectangle),
				new AnchorPoint(50, 100, false, true, MarkStyle.Rectangle),
				new AnchorPoint(100, 100, false, true, MarkStyle.Rectangle),
				new AnchorPoint(0, 50, true, true, MarkStyle.Custom)
			}, "rhombus2");
		}

		// assign anchor patterns to new nodes drawn by user
		diagram.addDiagramListener(new DiagramAdapter()
		{
			public void nodeCreated(NodeEvent e)
			{
				ShapeNode node = ((ShapeNode)e.getNode());
				if(diagram.getDefaultShape() == Shape.fromId("Rectangle"))
					node.setAnchorPattern(apat2);
				else
					node.setAnchorPattern(AnchorPattern.fromId("Decision1In3Out"));
			}
		});
	}

	private void createItems()
	{
		Shape decisionShape = com.mindfusion.diagramming.Shape.fromId("Decision");

		// create some nodes and links
		ShapeNode pb1 = diagram.getFactory().createShapeNode(10, 7, 25, 18);
		pb1.setShape(com.mindfusion.diagramming.Shape.fromId("Ellipse"));
		pb1.setText("Start");
		pb1.setAnchorPattern(apat1);

		ShapeNode pb2 = diagram.getFactory().createShapeNode(20, 75, 25, 18);
		pb2.setText("node 1");
		pb2.setAnchorPattern(apat2);

		ShapeNode pb3 = diagram.getFactory().createShapeNode(70, 70, 25, 18);
		pb3.setText("node 2");
		pb3.setAnchorPattern(apat2);

		ShapeNode pb4 = diagram.getFactory().createShapeNode(80, 100, 25, 18);
		pb4.setShape(com.mindfusion.diagramming.Shape.fromId("Ellipse"));
		pb4.setText("End");
		pb4.setAnchorPattern(apat1);

		ShapeNode decb1 = diagram.getFactory().createShapeNode(20, 35, 30, 20);
		decb1.getShape().getId().equals("Rhombus");
		decb1.setText("check 1");
		decb1.setShape(decisionShape);
		decb1.setAnchorPattern(AnchorPattern.fromId("Decision1In3Out"));

		ShapeNode decb2 = diagram.getFactory().createShapeNode(70, 30, 30, 20);
		decb2.setShape(com.mindfusion.diagramming.Shape.fromId("rhombusDef"));
		decb2.setText("check 2");
		decb2.setShape(decisionShape);
		decb2.setAnchorPattern(AnchorPattern.fromId("Decision2In2Out"));

		diagram.getFactory().createDiagramLink(decb1, decb2);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu)
	{
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}
}