package com.example.decisionlayout;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Queue;
import java.util.Random;

import com.mindfusion.diagramming.AnchorPattern;
import com.mindfusion.diagramming.DecisionLayout;
import com.mindfusion.diagramming.Diagram;
import com.mindfusion.diagramming.DiagramLink;
import com.mindfusion.diagramming.DiagramView;
import com.mindfusion.diagramming.Shape;
import com.mindfusion.diagramming.ShapeNode;

import com.mindfusion.drawing.Brush;
import com.mindfusion.drawing.Color;
import com.mindfusion.drawing.Font;
import com.mindfusion.drawing.GradientBrush;
import com.mindfusion.drawing.Pen;
import com.mindfusion.drawing.SolidBrush;

import android.os.Bundle;
import android.app.Activity;
import android.graphics.Typeface;
import android.view.Menu;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;

public class MainActivity extends Activity
{
	private Diagram diagram;
	private Brush terminalBrush;
	private Brush decisionBrush;
	private Brush normalBrush;

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		terminalBrush = new GradientBrush(new float[]{0.3f, 0.7f, 0.999f},
				new Color[]{
				Color.decode("#FFFFC0"),
				Color.decode("#BFBF90"),
				Color.decode("#FFFFC0")
		}, 90);
		decisionBrush = new GradientBrush(new float[]{0.3f, 0.7f, 0.999f},
				new Color[]{
				Color.decode("#F0FFFF"),
				Color.decode("#7FDF8F"),
				Color.decode("#F0FFFF")
		}, 90);
		normalBrush = new GradientBrush(new float[]{0.3f, 0.7f, 0.999f},
				new Color[]{
				Color.decode("#E0FFFF"),
				Color.decode("#A8BFBF"),
				Color.decode("#E0FFFF")
		}, 90);
		DiagramView view  = (DiagramView)findViewById(R.id.diag_view);
		diagram = view.getDiagram();
		diagram.setBackBrush(new SolidBrush(Color.white));
		
		Button btnLayoutRandom = (Button)findViewById(R.id.btn_random);
		btnLayoutRandom.setOnClickListener(new OnClickListener()
		{
			@Override
			public void onClick(View arg0) {
				layoutRandomDiagram();				
			}
		});
	}

	private void layoutRandomDiagram()
	{
		diagram.clearAll();
		
		ShapeNode startNode = diagram.getFactory().createShapeNode(0, 0, 30, 10);
		startNode.setBrush(terminalBrush);
		startNode.setFont(new Font("Arial", Typeface.BOLD, 4));
		startNode.setText("START");
		startNode.setId("start");
		startNode.setShape(Shape.fromId("Ellipse"));
		startNode.setPen(new Pen(1, Color.black));
		diagram.add(startNode);
		
		ShapeNode endNode = diagram.getFactory().createShapeNode(0, 0, 30, 10);
		endNode.setBrush(terminalBrush);
		endNode.setFont(new Font("Arial", Typeface.BOLD, 4));
		endNode.setText("END");
		endNode.setId("end");
		endNode.setShape(Shape.fromId("Ellipse"));
		endNode.setPen(new Pen(1, Color.black));
		diagram.add(endNode);
		
		final int maxLength = 10;
		ArrayList<ShapeNode> all = new ArrayList<ShapeNode>();
		Queue<ArrayList<ShapeNode>> remaining = new LinkedList<ArrayList<ShapeNode>>();
		
		ArrayList<ShapeNode> start = new ArrayList<ShapeNode>();
		start.add(startNode);
		
		remaining.add(start);
		
		Random random = new Random();
		int length = 0;
		while (remaining.size() > 0)
		{
			ArrayList<ShapeNode> newNodes = new ArrayList<ShapeNode>();
			boolean createdNew = false;
			
			ArrayList<ShapeNode> next = remaining.remove();
			for(int i = 0; i<next.size(); i++)
			{
				ShapeNode node = next.get(i);
				boolean isDecision = node.getTag() instanceof Boolean;
				
				ShapeNode previousChoice = null;
				int outLinks = isDecision ? 2 : 1;
				String[] linkTexts = isDecision ? new String[]{"YES", "NO"} : new String[]{""};
				for (int j = 0; j < outLinks; j++)
				{
					// Create new or link to an existing node;
					// Create at least one new node for each level
					boolean createNew = random.nextInt(maxLength) > length;
					if (!createdNew && i == next.size() - 1)
						createNew = true;
					if (length == maxLength)
						createNew = false;
					
					DiagramLink link;
					if (createNew)
					{
						// New decision or an operation node
						boolean newDecision = random.nextInt(3) > 0 && !isDecision;
						ShapeNode newNode = diagram.getFactory().createShapeNode(0,0,30,10);
						if (newDecision)
						{
							newNode.setShape(Shape.fromId("Decision"));
							newNode.setAnchorPattern(AnchorPattern.fromId("Decision2In2Out"));
							newNode.setBrush(decisionBrush);
							newNode.setText("Decision");
							newNode.setTag(true);
						}
						else
						{
							newNode.setShape(Shape.fromId("Rectangle"));
							newNode.setBrush(normalBrush);
							newNode.setText("Operation");
						}
						
						link = diagram.getFactory().createDiagramLink(node, newNode);
						newNodes.add(newNode);
						createdNew = true;
					}
					else
					{
						// Link to an existing node. If length == maxLength, link to the end node
						if (length == maxLength || remaining.size() == 0)
						{
							link = diagram.getFactory().createDiagramLink(node, endNode);
						}
						else
						{
							// Make sure both choices don't lead to the same node
							ShapeNode choice = null;
							while (choice == previousChoice)
							{
								choice = all.get(random.nextInt(all.size()));
							}
							link = diagram.getFactory().createDiagramLink(node, choice);
							previousChoice = choice;
						}
					}
					link.setText(linkTexts[j]);
				}
				all.add(node);
			}
			if (newNodes.size()>0)
				remaining.add(newNodes);
			
			length++;
		}
		arrangeDiagram();
	}
	
	private void arrangeDiagram()
	{
		DecisionLayout layout = new DecisionLayout();
		layout.setHorizontalPadding(10);
		layout.setVerticalPadding(10);
		layout.setStartNode(diagram.findNodeById("start"));
		layout.arrange(diagram);
	}
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu)
	{
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}
}