package com.mindfusion.tutorial2;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.mindfusion.tutorial2.R;
import com.mindfusion.diagramming.Diagram;
import com.mindfusion.diagramming.DiagramNode;
import com.mindfusion.diagramming.DiagramView;
import com.mindfusion.diagramming.ShapeNode;
import com.mindfusion.diagramming.TreeLayout;
import com.mindfusion.diagramming.TreeLayoutLinkType;
import com.mindfusion.diagramming.TreeLayoutType;
import com.mindfusion.diagramming.jlayout.TreeLayoutDirection;

import com.mindfusion.drawing.Color;
import com.mindfusion.drawing.SolidBrush;

import android.os.Bundle;
import android.app.Activity;
import android.content.Context;
import android.graphics.RectF;


public class MainActivity extends Activity
{
	private Diagram diagram;
	
	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
		DiagramView view  = (DiagramView)findViewById(R.id.diag_view);
		diagram = view.getDiagram();
		diagram.setBackBrush(new SolidBrush(Color.white));
		
		loadTree(diagram, "SampleTree.xml", this);
	}

	final RectF bounds = new RectF(0,0,30,8);
	
	void createChildren(Diagram diagram, DiagramNode parentNode, Node parentElement)
	{
		NodeList elements = parentElement.getChildNodes();
		for(int i = 0; i<elements.getLength(); i++)
		{
			Node element = elements.item(i);
			if(element.getNodeName().equals("Activity"))
			{
				ShapeNode node = diagram.getFactory().createShapeNode(bounds);
				node.setText(((Element)element).getAttribute("Name"));
				diagram.getFactory().createDiagramLink(parentNode, node);
				createChildren(diagram, node, element);
			}
		}
	}
	
	void loadTree(Diagram diagram, String fileName, Context context)
	{
		try
		{
			// load the tree xml
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document xml = builder.parse(
				getAssets().open(fileName));
			Element rootElement = (Element)xml.getFirstChild();

			// create diagram elements recursively
			ShapeNode root = diagram.getFactory().createShapeNode(bounds);
			root.setText("Project");
			createChildren(diagram, root, rootElement);
	
			// arrange using TreeLayout
			TreeLayout layout = new TreeLayout();
			layout.setType(TreeLayoutType.Cascading);
			layout.setDirection(TreeLayoutDirection.LeftToRight);
			layout.setLinkStyle(TreeLayoutLinkType.Cascading2);
			layout.setNodeDistance(5);
			layout.setLevelDistance(-5);
			layout.arrange(diagram);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}
}